import logging
from flask import Flask

from connexion import FlaskApp

class App(FlaskApp):
    def __init__(self, name, use_tracer=None, use_metric=False, use_logging_level=logging.DEBUG, use_optimizer=None, use_cors=None, all=False):
        """
        Initialize Flask App with multiple microservice and easier usability features.
        """
        super().__init__(name)
        logger = logging.getLogger("")

        self.metrics = None
        self.tracing = None
        self.optimize = None
        self.cors = None

        logger.info("--- Start Connexion-Plus ---")

        if not isinstance(self.app, (Flask, FlaskApp)):
            logger.warning("Given App is not flask, so it cannot get any functionality added from this lib currently.")
            return

        # add optimizer
        if use_optimizer is not None and use_optimizer is not False:
            from .Optimizer import FlaskOptimize
            
            config = {"compress": True, "minify": True}
            if isinstance(use_optimizer, dict):
                config = use_optimizer

            self.optimize = FlaskOptimize(self.app, config)
            logger.info("Optimizer added.")

        # add CORS
        if use_cors is not None and use_cors is not False:
            from flask_cors import CORS

            if isinstance(use_cors, dict):
                self.cors = CORS(self.app, resources=use_cors)
            else:
                self.cors = CORS(self.app)
            
            logger.info("CORS added.")

        # add prometheus
        if use_metric is not None and use_metric is not False:
            # TODO: add configuration https://github.com/rycus86/prometheus_flask_exporter#configuration

            from prometheus_flask_exporter import PrometheusMetrics
            self.metrics = PrometheusMetrics(self.app)
            logger.info("Prometheus added.")

        # add tracing
        if use_tracer is not None and use_tracer is not False:
            # add tracing to all routes in flaskApp
            from flask_opentracing import FlaskTracing
            import opentracing

            config = None
            if not isinstance(use_tracer, opentracing.Tracer):
                logger.info("Tracer should be used, but no object was given.")
                from jaeger_client import Config as jConfig

                if isinstance(use_metric, bool) and use_metric is True:
                    logger.info("Use metrics for tracer.")
                    from jaeger_client.metrics.prometheus import PrometheusMetricsFactory
                    config = jConfig(
                        config={},
                        service_name="MicroserviceConnexionPlus",
                        metrics_factory=PrometheusMetricsFactory(namespace="MicroserviceConnexionPlus")
                    )
                else:
                    config = jConfig(
                        config={},
                        service_name="MicroserviceConnexionPlus",
                    )
            else:
                logger.info("Tracer given and will be used.")
            
            tracer_obj = use_tracer if config is None else config.initialize_tracer()
            self.tracing = FlaskTracing(tracer_obj, True, self.app)
            
            # add tracer to everything to support spans through multiple microservices via rpc-calls
            from opentracing_instrumentation.client_hooks import install_all_patches
            install_all_patches()
            logger.info("All tracing relevant libs patched.")

            # add a TracingHandler for Logging
            from .TracingHandler import TracingHandler

            th = TracingHandler(use_tracer)
            th.setLevel(use_logging_level)

            logging.getLogger('').addHandler(th)
            logger.info("Tracer added.")

        logger.info("--- Finished Connexion-Plus ---")




import re, string
import sys
from connexion.resolver import RestyResolver, Resolution
from connexion.exceptions import ResolverError

class MultipleResourceResolver(RestyResolver):
    def resolve(self, operation):
        """
        Default operation resolver
        :type operation: connexion.operations.AbstractOperation
        """

        # set randomizer to not collide endpoint_names, so that parameters for resources are possible.
        # Otherwise the following example collides:
        #   /res1 [GET] and /res1/{id} [GET]
        operation._randomize_endpoint = 2

        operation_id = self.resolve_operation_id(operation)
        try:
            return Resolution(self.resolve_function_from_operation_id(operation_id), operation_id)
        except ResolverError:
            return Resolution(self.resolve_function_from_operation_id(operation_id.lower()), operation_id)
            

    def resolve_operation_id_using_rest_semantics(self, operation):
        """
        Resolves the operationId using REST semantics without collision for longer paths with multiple ressources
        :type operation: connexion.operations.AbstractOperation
        """
        path_match = re.search(
            r'^/?(?P<resource_name>([\w\-](?<!/))*)(?P<trailing_slash>/*)(?P<extended_path>.*)$', operation.path
        )

        count_resource = 0
        count_parameters = 0

        def get_controller_name():
            nonlocal count_parameters, count_resource
            x_router_controller = operation.router_controller

            #chars = re.escape(string.punctuation)
            #resource_name = re.sub(r'[' + chars + ']', '_', operation.path)

            # empty name to append to

            name = self.default_module_name
            resource_name = ''
            # split path at slash to separate every parameter
            split = re.split("/", operation.path)
            for s in split:
                # find the parameter, where a variable was defined to exlude it in resource_name
                pattern = re.compile(r"\{[a-zA-Z-_]+\}")
                if s and pattern.search(s) is None:
                    count_resource += 1 # count up the founded resource
                    resource_name += s.title()

            if x_router_controller:
                name = x_router_controller

            elif resource_name:
                resource_controller_name = resource_name.replace('-', '_')
                name += '.' + resource_controller_name

            # calc parameters count
            count_parameters = len(split) - count_resource

            return name

        def get_function_name():
            nonlocal count_parameters, count_resource
            method = operation.method

            is_collection_endpoint = \
                method.lower() == 'get' \
                and count_resource > count_parameters # if more resources than parameters, the user wants to use search

            return self.collection_endpoint_name if is_collection_endpoint else method.lower()

        return '{}.{}'.format(get_controller_name(), get_function_name())

